import { deepAssign } from '@codeperate/utils';
import { StringWidgetProps } from '../StringWidget';
import { TextAreaWidgetProps } from '../TextAreaWidget';
import { PasswordWidgetProps } from '../PasswordWidget';
import { ArrayWidgetContext } from '../ArrayWidget';
import { NumberWidgetProps } from '../NumberWidget';
import { DateWidgetProps } from '../DateWidget';
import { SectionWidgetProps } from '../SectionWidget';
import { ObjectWidgetProps } from '../ObjectWidget';
import { FileWidgetProp } from '../FileWidget';
import { CheckBoxWidgetProps } from '../CheckBoxWidget';

type ToDefault<T extends object> = {
    [K in keyof T]: T[K] extends object ? (T[K] extends Function ? T[K] : () => T[K]) : T[K];
};
export interface ComponentsConfig {
    [key: string]: any;
    ObjectWidget: ToDefault<ObjectWidgetProps>;
    StringWidget: ToDefault<StringWidgetProps>;
    TextAreaWidget: ToDefault<TextAreaWidgetProps>;
    PasswordWidget: ToDefault<PasswordWidgetProps>;
    ArrayWidget: ToDefault<ArrayWidgetContext>;
    NumberWidget: ToDefault<NumberWidgetProps>;
    DateWidget: ToDefault<DateWidgetProps>;
    SectionWidget: ToDefault<SectionWidgetProps>;
    FileWidget: ToDefault<FileWidgetProp>;
    CheckBoxWidget: ToDefault<CheckBoxWidgetProps>;
}

const config: ComponentsConfig = {} as ComponentsConfig;

export function useConfig<T extends keyof ComponentsConfig, K>(key: T, defaultValue?: K) {
    return deepAssign(defaultValue ?? {}, config?.[key] ?? {}) as K;
}

export function setConfig<T extends keyof ComponentsConfig>(key: T, value: ComponentsConfig[T]) {
    config[key] = value;
}
